import org.apache.lucene.timeslots.TSIndexer;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.File;


public class TSIndexerMain 
{
	private static void usage()
	{
		System.out.println("Usage: TSIndexerMain -index-path path OPTIONS");
		System.out.println("OPTIONS:");
		System.out.println("  -doc-path-list-to-index listfile");
		System.out.println("  -create-new-index");
		System.out.println("  -optimize");
	}

	public static void main(String[] args) throws IOException
	{
        String arg;
		String indexPath = null;
		String docPathListToIndex = null;
		boolean doOptimize = false;
		boolean doCreateNewIndex = false;
		int i = 0;
		while(i < args.length && args[i].startsWith("-"))
		{
            arg = args[i++];

			if (arg.equals("-index-path")) 
			{
				if (i < args.length)
					indexPath = args[i++];
				else System.err.println("-index-path requires a filename");
			}
			else if (arg.equals("-doc-path-list-to-index")) 
			{
				if (i < args.length)
					docPathListToIndex = args[i++];
				else System.err.println("-doc-path-list-to-index requires a filename");
			}
			else if (arg.equals("-create-new-index")) 
			{
				doCreateNewIndex = true;
			}
			else if (arg.equals("-optimize")) 
			{
				doOptimize = true;
			}
		}
		if (indexPath==null) {
			usage();
			return;
		}
		System.out.println("Application parameters: -index-path '"+indexPath+"' -doc-path-list-to-index '"+docPathListToIndex+"'");

		TSIndexer indexer = new TSIndexer(indexPath, doCreateNewIndex);

		if (docPathListToIndex != null)
		{
			BufferedReader input =  new BufferedReader(new FileReader(new File(docPathListToIndex)));
			String line = null; //not declared within while loop
			int line_number = 0;
			while (( line = input.readLine()) != null){
				line_number++;
				String [] a = line.split("[ \t]+");
				if (a.length != 3) {
					System.err.println("Error in the index document list on line "+line_number);
					System.err.println("Expected format is: DOCUMENT_NAME DOCUMENT_PATH DOCUMENT_LENGTH_IN_SECONDS");
					break;
				}
				float length_seconds = Float.valueOf(a[2]).floatValue();
				indexer.index(a[0], a[1], length_seconds);
			}
			indexer.commit();
		}

		if (doOptimize)
		{
			indexer.optimize();
		}

		indexer.close();
	}
}
