import org.apache.lucene.timeslots.TSSearcher;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.LinkedList;
import java.nio.charset.Charset;


public class TSSearcherMain 
{
	private static void usage()
	{
		System.out.println("Usage: TSSearcherMain -index-path path -query query_text");
	}

	public static void main(String[] args) throws IOException
	{
        String arg;
		String indexPath = null;
		String query = null;
		String stopwordslist = null;
		LinkedList<String> docs = null;
		boolean doclistonly = false;
		float maxworddistance = 0.0f;
		String wordsmap = null;
		String clusterslist = null;
		boolean explainresults = false;
		int docpagesize = 10;
		int docpagenumber = 1;
		int occpagesize = 10;
		int occpagenumber = 1;
		boolean sortoccsbyscore = false;
/*
		System.out.println("Application parameters:");
		for(int i=0; i<args.length; i++) 
		{
			System.out.println("  "+args[i]);
		}
*/
		int i = 0;
		while(i < args.length && args[i].startsWith("-"))
		{
            arg = args[i++];

			if (arg.equals("-index-path")) 
			{
				if (i < args.length)
					indexPath = args[i++];
				else System.err.println("-index-path requires a filename");
			}
			else if (arg.equals("-query")) 
			{
				if (i < args.length)
					query = args[i++];
				else System.err.println("-query requires a query text");
			}
			else if (arg.equals("-docs")) 
			{
				if (i < args.length) {
					String[] a = args[i++].split(",");
					docs = new LinkedList<String>();
					System.out.println("a.length: "+a.length);
					for (int j=0; j<a.length; j++) {
						System.out.println("a["+j+"]: "+a[j]);
						docs.add(a[j]);
					}
				}
				else System.err.println("-docs requires a comma-separated list of documents");
			}
			else if (arg.equals("-stopwords-list")) 
			{
				if (i < args.length)
					stopwordslist = args[i++];
				else System.err.println("-stopwordslist requires a filename");
			}
			else if (arg.equals("-doclist-only")) 
			{
				doclistonly = true;
			}
			else if (arg.equals("-max-words-distance")) 
			{
				if (i < args.length) {
					maxworddistance = Float.valueOf(args[i++]).floatValue();
					System.out.println("maxworddistance: "+maxworddistance);
				}
				else System.err.println("-maxworddistance requires time in seconds");
			}
			else if (arg.equals("-words-map")) 
			{
				if (i < args.length)
					wordsmap = args[i++];
				else System.err.println("-words-map requires a filename");
			}
			else if (arg.equals("-clusters-list")) 
			{
				if (i < args.length)
					clusterslist = args[i++];
				else System.err.println("-clusters-list requires a filename");
			}
			else if (arg.equals("-explain-results")) 
			{
				explainresults = true;
			}
			else if (arg.equals("-doc-page-size")) 
			{
				if (i < args.length)
					docpagesize = Integer.parseInt(args[i++]);
				else System.err.println("-doc-page-size requires an integer");
			}
			else if (arg.equals("-doc-page-number")) 
			{
				if (i < args.length)
					docpagenumber = Integer.parseInt(args[i++]);
				else System.err.println("-doc-page-number requires an integer");
			}
			else if (arg.equals("-occ-page-size")) 
			{
				if (i < args.length)
					occpagesize = Integer.parseInt(args[i++]);
				else System.err.println("-occ-page-size requires an integer");
			}
			else if (arg.equals("-occ-page-number")) 
			{
				if (i < args.length)
					occpagenumber = Integer.parseInt(args[i++]);
				else System.err.println("-occ-page-number requires an integer");
			}
			else if (arg.equals("-sort-occurrences-by-score")) 
			{
				sortoccsbyscore = true;
			}
			else {
				System.err.println("ERROR: unrecognized option '"+arg+"'");
				usage();
				return;
			}
		}
		if (indexPath==null || query==null) {
			usage();
			return;
		}

		PrintWriter out = new PrintWriter(System.out, true);

		out.println("Charset.defaultCharset(): "+Charset.defaultCharset());

		TSSearcher searcher = new TSSearcher(indexPath, 0.5f);
		if (stopwordslist != null) {
			searcher.loadStopWordsList(stopwordslist);
		}
		if (wordsmap != null) {
			searcher.loadWordsMap(wordsmap);
		}
		if (clusterslist != null) {
			searcher.loadWordClusters(clusterslist);
		}
		searcher.setDoExplainResults(explainresults);
		searcher.search(query, docs, doclistonly, docpagesize, docpagenumber, occpagesize, occpagenumber, sortoccsbyscore, maxworddistance, out);

		searcher.close();
	}
}
