import org.apache.lucene.timeslots.TSSearcher;
//import org.apache.lucene.queryParser.ParseException;
import java.io.IOException;
import java.io.*;
import java.util.LinkedList;
import javax.servlet.*;
import javax.servlet.http.*;

public class TSSearcherServlet extends HttpServlet
{
	private static final long serialVersionUID = 7L;
	private int counter = 0;
	private TSSearcher searcher;

	public void init() throws ServletException {
		String indexPath = getServletConfig().getInitParameter("indexPath");
		getServletContext().log("indexPath: "+indexPath); 

		String clustersFile = getServletConfig().getInitParameter("clustersFile");
		getServletContext().log("clustersFile: "+clustersFile); 

		String wordMapFile = getServletConfig().getInitParameter("wordMapFile");
		getServletContext().log("wordMapFile: "+wordMapFile); 

		String stopWordsFile = getServletConfig().getInitParameter("stopWordsFile");
		getServletContext().log("stopWordsFile: "+stopWordsFile); 

		String slotlen_str = getServletConfig().getInitParameter("timeSlotLength");
		float slotlen = Float.valueOf(slotlen_str).floatValue();
		getServletContext().log("slotlen_str: "+slotlen_str); 

		try {
			searcher = new TSSearcher(indexPath, slotlen);
			if (clustersFile != null) {
				searcher.loadWordClusters(clustersFile);
			}
			if (wordMapFile != null) {
				searcher.loadWordsMap(wordMapFile);
			}
			if (stopWordsFile != null) {
				searcher.loadStopWordsList(stopWordsFile);
			}
		} catch (IOException e) {
			throw new ServletException();
		}
	}

	public void destroy() {
		try {
			searcher.close();
		} catch (IOException e) {
			getServletContext().log("An exception occurred in TSSearcherServlet::destroy()", e); 
		}
	}

    protected void doGet(HttpServletRequest req, HttpServletResponse resp)
        throws ServletException, IOException
    {
//		PrintWriter out = new PrintWriter(new OutputStreamWriter(resp.getOutputStream(), "UTF-8"), true);
		resp.setContentType("text/xml;charset=UTF-8");
        PrintWriter out = resp.getWriter();

        String query = req.getParameter("q");
		boolean doclistonly = req.getParameter("doclistonly") != null ? true : false;

		String docpagesize_str = req.getParameter("dps");
		int docpagesize = docpagesize_str != null ? Integer.parseInt(docpagesize_str) : 10;

		String docpagenumber_str = req.getParameter("dpn");
		int docpagenumber = docpagenumber_str != null ? Integer.parseInt(docpagenumber_str) : 1;

		String occpagesize_str = req.getParameter("ops");
		int occpagesize = occpagesize_str != null ? Integer.parseInt(occpagesize_str) : 10;

		String occpagenumber_str = req.getParameter("opn");
		int occpagenumber = occpagenumber_str != null ? Integer.parseInt(occpagenumber_str) : 1;

		String maxworddistanceseconds_str = req.getParameter("dist");
		float maxworddistanceseconds = maxworddistanceseconds_str != null ? Float.valueOf(maxworddistanceseconds_str).floatValue() : 1.0f;

		boolean sortoccsbyscore = req.getParameter("sortsc") != null;

		String docs_str = req.getParameter("docs");
		LinkedList<String> docs = null;
		if (docs_str != null) {
			String[] docs_a = docs_str.split(",");
			docs = new LinkedList<String>();
			for (String doc : docs_a) {
				docs.add(doc);
			}
		}

		searcher.search(query, docs, doclistonly, docpagesize, docpagenumber, occpagesize, occpagenumber, sortoccsbyscore, maxworddistanceseconds, out);

		counter++;
	}
}

