package org.apache.lucene.timeslots;

import org.apache.lucene.analysis.TokenFilter;
import org.apache.lucene.analysis.TokenStream;
import org.apache.lucene.analysis.tokenattributes.PayloadAttribute;
import org.apache.lucene.analysis.tokenattributes.TermAttribute;
import org.apache.lucene.analysis.tokenattributes.PositionIncrementAttribute;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.WhitespaceTokenizer;
import org.apache.lucene.analysis.LowerCaseFilter;
import org.apache.lucene.analysis.payloads.PayloadEncoder;
import org.apache.lucene.analysis.payloads.FloatEncoder;
import org.apache.lucene.analysis.payloads.PayloadHelper;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.IndexWriter;
//import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.Term;
import org.apache.lucene.index.TermVectorMapper;
import org.apache.lucene.index.TermVectorOffsetInfo;
//import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.DefaultSimilarity;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.TopDocs;
import org.apache.lucene.search.spans.Spans;
import org.apache.lucene.search.payloads.PayloadTermQuery;
import org.apache.lucene.search.payloads.PayloadNearQuery;
//import org.apache.lucene.search.payloads.AveragePayloadFunction;
import org.apache.lucene.search.payloads.MinPayloadFunction;
//import org.apache.lucene.search.payloads.PayloadFunction;
import org.apache.lucene.store.Directory;
//import org.apache.lucene.store.RAMDirectory;
import org.apache.lucene.store.NIOFSDirectory;


import java.io.IOException;
import java.io.Reader;
import java.io.FileReader;
import java.io.File;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Collection;
import java.util.Iterator;
import java.util.ArrayList;

public class LuceneExperiment {

	public static String[] DOCS = {
		"lattices/BIN-2008-02-05_0000789_0000943.lat.mesh.txt",
		"lattices/BIN-2008-02-05_0001071_0001158.lat.mesh.txt",
		"lattices/BIN-2008-02-05_0001220_0001409.lat.mesh.txt",
		"lattices/BIN-2008-02-05_0001409_0001723.lat.mesh.txt",
		"lattices/BIN-2008-02-05_0001760_0002440.lat.mesh.txt",
		"lattices/BIN-2008-02-05_0002488_0003377.lat.mesh.txt",
		"lattices/BIN-2008-02-05_0003444_0003544.lat.mesh.txt",
		"lattices/BIN-2008-02-05_0004247_0004306.lat.mesh.txt",
		"lattices/BIN-2008-02-05_0004460_0006065.lat.mesh.txt",
		"lattices/BIN-2008-02-05_0006087_0006835.lat.mesh.txt",
		"lattices/BIN-2008-02-05_0006842_0006910.lat.mesh.txt",
		"lattices/BIN-2008-02-05_0006912_0007214.lat.mesh.txt",
		"lattices/BIN-2008-02-05_0007234_0007324.lat.mesh.txt"
	};

	// OLD INDICES {{{
/*	
	public static String[] DOCS = {
		"15|TAKTEZ|0.000981727|7.89|0.38 15|TAKHLE|0.00134338|7.89|0.35 15|TAK|0.961188|7.89|0.3 16|ZATIM|0.00347171|8.2|0.21 16|TO|0.31264|8.28|0.07 16|BYCH|0.000248472|8.2|0.14",
		"17|DOVOPRAVDY|0.230652|8.59|0.2 17|DOPRAVILI|0.000478271|8.44|0.28 17|MIT|0.00887974|8.73|0.43 17|DOOPRAVDY|0.696953|8.97|0.11 18|E.|0.135026|8.73|0.56 18|A|0.0106019|9.26|0.03 18|JI|0.0517104|9.05|0.24"
	};
*/
/*	
	public static String[] DOCS = {
		"The quick|2.0 red|2.0 fox|10.0 jumped|5.0 over the lazy|2.0 brown|2.0 dogs|10.0",
		"The quick red fox jumped over the lazy brown dogs",//no boosts
		"The quick|2.0 red|2.0 fox|10.0 jumped|5.0 over the old|2.0 brown|2.0 box|10.0",
		"Mary|10.0 had a little|2.0 lamb|10.0 whose fleece|10.0 was|5.0 white|2.0 as snow|10.0",
		"Mary had a little lamb whose fleece was white as snow",
		"Mary|10.0 takes on Wolf|10.0 Restoration|10.0 project|10.0 despite ties|10.0 to sheep|10.0 farming|10.0",
		"Mary|10.0 who lives|5.0 on a farm|10.0 is|5.0 happy|2.0 that she|10.0 takes|5.0 a walk|10.0 every day|10.0",
		"Moby|10.0 Dick|10.0 is|5.0 a story|10.0 of a whale|10.0 and a man|10.0 obsessed|10.0",
		"The robber|10.0 wore|5.0 a black|2.0 fleece|10.0 jacket|10.0 and a baseball|10.0 cap|10.0",
		"The English|10.0 Springer|10.0 Spaniel|10.0 is|5.0 the best|2.0 of brown|1.0 dogs|10.0"
	};
*/
/*
	public static String[] DOCS = {
		"The quick red fox jumped over the lazy brown dogs.",
		"Mary had a little lamb whose fleece was white as snow.",
		"Moby Dick is a story of a whale and a man obsessed.",
		"The robber wore a black fleece jacket and a baseball cap.",
		"The English Springer Spaniel is the best of all dogs.",
	};
*/
	// }}}
	
	protected PayloadSimilarity payloadSimilarity;

	protected float timeSlotLength = 0.5f;

	public void run() throws IOException
	{
		PayloadAnalyzer analyzer = new PayloadAnalyzer();

		// 1. create the index
		//Directory index = new RAMDirectory();
		Directory index = new NIOFSDirectory(new File("./index"));

		// the boolean arg in the IndexWriter ctor means to
		// create a new index, overwriting any existing index
		IndexWriter w = new IndexWriter(index, analyzer, true, IndexWriter.MaxFieldLength.UNLIMITED);

		payloadSimilarity = new PayloadSimilarity();
		w.setSimilarity(payloadSimilarity);

		for (int i = 0; i < DOCS.length; i++) {
			System.out.println("indexing '"+DOCS[i]+"'");
			Document doc = new Document();
			Field id = new Field("id", "doc_" + i, Field.Store.YES, Field.Index.NOT_ANALYZED_NO_NORMS);
			doc.add(id);
			//Store both position and offset information
//			Field text = new Field("body", DOCS[i], Field.Store.NO, Field.Index.ANALYZED, Field.TermVector.WITH_POSITIONS_OFFSETS);
			File f = new File(DOCS[i]);
			Field text = new Field("body", new FileReader(f), Field.TermVector.WITH_POSITIONS_OFFSETS);
			//Field text = new Field("body", new FileReader(f), Field.Store.NO, Field.Index.ANALYZED, Field.TermVector.WITH_POSITIONS_OFFSETS);
			doc.add(text);
			w.addDocument(doc);
		}
		w.close();




		// the "title" arg specifies the default field to use
		// when no field is explicitly specified in the query.
		//    Query q = new QueryParser("content", analyzer).parse(querystr);

		// 3. search
		IndexSearcher searcher = new IndexSearcher(index, true);
		searcher.setSimilarity(payloadSimilarity);

		TopDocs topDocs;
/*
		PayloadTermQuery ptq = new PayloadTermQuery(new Term("body", "fox"), new AveragePayloadFunction());
		topDocs = searcher.search(ptq, 10);
		printResults(searcher, ptq, topDocs);

		TermQuery tq = new TermQuery(new Term("body", "fox"));
		topDocs = searcher.search(tq, 10);
		printResults(searcher, tq, topDocs);
*/
		// phrase "brown dogs"
		PayloadTermQuery clauses[] = new PayloadTermQuery[2];
		clauses[0] = new PayloadTermQuery(new Term("body", "dopravy"), new MyMinPayloadFunction(), false);
		clauses[1] = new PayloadTermQuery(new Term("body", "pravdy"), new MyMinPayloadFunction(), false);
		PayloadNearQuery q = new PayloadNearQuery(clauses, 2, true, new MyMinPayloadFunction());
		
//		PayloadTermQuery q = new PayloadTermQuery(new Term("body", "pravdy"), new MyMinPayloadFunction(), false);
		topDocs = searcher.search(q, 10);
		printResults(searcher, q, topDocs);







//		PayloadTermQuery q = new PayloadTermQuery(new Term("body", "fleece"), new AveragePayloadFunction());
		topDocs = searcher.search(q, 10);
		for (int i = 0; i < topDocs.scoreDocs.length; i++) {
			ScoreDoc scoreDoc = topDocs.scoreDocs[i];
			System.out.println("Score Doc: " + scoreDoc);
		}

		//IndexReader reader = searcher.getIndexReader();
		Spans spans = q.getSpans(searcher.getIndexReader());
		while (spans.next() == true){
			Collection<byte[]> payloads = spans.getPayload();
			System.out.println("payloads.size():"+payloads.size());

			float spanScore = 1;
			for (Iterator<byte[]> iterator = payloads.iterator(); iterator.hasNext();) {
				byte[] thePayload = iterator.next();
				float payloadScore = PayloadHelper.decodeFloat(thePayload, 0);
				if (spanScore > payloadScore) {
					spanScore = payloadScore;
				}
				System.out.println("payload:"+payloadScore);
			}
			
//			System.out.println("payload: " + (PayloadHelper.decodeFloat(payloads.get(0), 0)));
			System.out.println("Doc: " + spans.doc() + " Start: " + (spans.start()*timeSlotLength) + " End: " + (spans.end()*timeSlotLength) + " Score: "+spanScore);
		}
/*
		// PRINT CONTEXT
		WindowTermVectorMapper tvm = new WindowTermVectorMapper();
		int window = 2;//get the words within two of the match, inclusive of the boundaries
		while (spans.next() == true) {
			System.out.println("Doc: " + spans.doc() + " Start: " + spans.start() + " End: " + spans.end());
			//build up the window
			tvm.start = spans.start() - window;
			tvm.end = spans.end() + window;
			reader.getTermFreqVector(spans.doc(), "body", tvm);
			for (WindowEntry entry : tvm.entries.values()) {
				System.out.println("Entry: " + entry);
			}
			//clear out the entries for the next round
			tvm.entries.clear();
		}
*/
		// searcher can only be closed when there
		// is no need to access the documents any more. 
		searcher.close();
	}

	private void printResults(IndexSearcher searcher, Query query, TopDocs topDocs) throws IOException {
		System.out.println("-----------");
		System.out.println("Results for " + query + " of type: " + query.getClass().getName());
		for (int i = 0; i < topDocs.scoreDocs.length; i++) {
			ScoreDoc doc = topDocs.scoreDocs[i];
			System.out.println("Doc: " + doc.toString());
			System.out.println("Explain: " + searcher.explain(query, doc.doc));
		}
	}
}

class PayloadAnalyzer extends Analyzer {
	public TokenStream tokenStream(String fieldName, Reader reader) {
		TokenStream result = new WhitespaceTokenizer(reader);
		result = new LowerCaseFilter(result);
		result = new TimeSlotsTokenFilter(result);
		return result;
	}
}

class MyMinPayloadFunction extends MinPayloadFunction {
	private static final long serialVersionUID = 5L;

	@Override
	public float currentScore(int docId, String field, int start, int end, int numPayloadsSeen, float currentScore, float currentPayloadScore) {
//		System.out.println("MyMinPayloadFunction: numPayloadsSeen="+numPayloadsSeen+" currentPayloadScore="+currentPayloadScore+" currentScore="+currentScore+" return="+(numPayloadsSeen > 0 ? Math.min(currentPayloadScore, currentScore) : currentPayloadScore));
		return numPayloadsSeen > 0 ? Math.min(currentPayloadScore, currentScore) : currentPayloadScore;
	}
}


class TimeSlotsTokenFilter extends TokenFilter 
{
	public static final char DEFAULT_DELIMITER = '|';
	protected char delimiter = DEFAULT_DELIMITER;
	protected TermAttribute termAtt;
	protected PayloadAttribute payAtt;
	protected PositionIncrementAttribute posAtt;
	protected PayloadEncoder floatEncoder;
	protected int currentTimeSlot = 0;

	/**
	 * Construct a token stream filtering the given input.
	 */
	protected TimeSlotsTokenFilter(TokenStream input) {
		super(input);
		floatEncoder = new FloatEncoder();
		termAtt = addAttribute(TermAttribute.class);
		payAtt = addAttribute(PayloadAttribute.class);
		posAtt = addAttribute(PositionIncrementAttribute.class);
	}

	public boolean incrementToken() throws IOException {
		boolean result = false;
		if (input.incrementToken()) {
			final char[] buffer = termAtt.termBuffer();
			final int length = termAtt.termLength();
			//look for the delimiter
			int timeslot = 0;
			int i=0;
			// READ THE TIME SLOT INFO
			while (i < length) {
				if (buffer[i] == delimiter) {
					String s = new String(buffer, 0, i);
					timeslot = Integer.parseInt(s);
//					System.out.println("Slot: " + s + " = " + timeslot + " .. " + (timeslot-currentTimeSlot));
					posAtt.setPositionIncrement(timeslot - currentTimeSlot);
					currentTimeSlot = timeslot;
					break;//at this point, we know the whole piece, so we can exit.  If we don't see the delimiter, then the termAtt is the same
				}
				i++;
			}
			// READ THE TERM TEXT
			int item_beg = ++i;
			while (i < length) {
				if (buffer[i] == delimiter) {
/*					String s = new String(buffer);
					s = s.substring(item_beg, i);
					System.out.println("Term: " + s);
*/					termAtt.setTermBuffer(buffer, item_beg, i-item_beg);
					break;
				}
				i++;
			}
			// READ THE TERM'S POSTERIOR PROBABILITY
			item_beg = ++i;
			int item_end = length;
			while (i < length) {
				if (buffer[i] == delimiter) {
					item_end = i;
					break;
				}
				i++;
			}
/*			String s = new String(buffer);
			s = s.substring(item_beg, item_end);
			System.out.println("Score: " + s);
*/			payAtt.setPayload(floatEncoder.encode(buffer, item_beg, item_end-item_beg));

			result = true;
		}
		return result;
	}
}


class PayloadSimilarity extends DefaultSimilarity {
	private static final long serialVersionUID = 6L;

	@Override
		public float scorePayload(int docId, String fieldName, int start, int end, byte[] payload, int offset, int length) {
//			System.out.println("PayloadSimilarity::scorePayload = "+PayloadHelper.decodeFloat(payload, offset));
			return PayloadHelper.decodeFloat(payload, offset);//we can ignore length here, because we know it is encoded as 4 bytes
		}
}

//Not thread-safe
class WindowTermVectorMapper extends TermVectorMapper {
 
  int start;
  int end;
  LinkedHashMap<String, WindowEntry> entries = new LinkedHashMap<String, WindowEntry>();
 
  public void map(String term, int frequency, TermVectorOffsetInfo[] offsets, int[] positions) {
    for (int i = 0; i < positions.length; i++) {//unfortunately, we still have to loop over the positions
      //we'll make this inclusive of the boundaries
      if (positions[i] >= start && positions[i] < end){
        WindowEntry entry = entries.get(term);
        if (entry == null) {
          entry = new WindowEntry(term);
          entries.put(term, entry);
        }
        entry.positions.add(positions[i]);
      }
    }
  }
 
  public void setExpectations(String field, int numTerms, boolean storeOffsets, boolean storePositions) {
    // do nothing for this example
    //See also the PositionBasedTermVectorMapper.
  }
 
}
 
class WindowEntry{
  String term;
  List<Object> positions = new ArrayList<Object>();//a term could appear more than once w/in a position
 
  WindowEntry(String term) {
    this.term = term;
  }
 
  @Override
  public String toString() {
    return "WindowEntry{" +
            "term='" + term + '\'' +
            ", positions=" + positions +
            '}';
  }
}
