package org.apache.lucene.timeslots;


import org.apache.lucene.search.spans.Spans;
import java.util.HashMap;
import java.io.IOException;


class TSDoc2OccurrencesCount extends HashMap<Integer, Integer>
{
	private static final long serialVersionUID = 8L;

	/** Accumulate counts without taking care of overlapping occurences.
	 */
	public void fill_approximate(Spans spans) throws IOException
	{
		int curDoc = -1;
		Integer occCount = null;
		while (spans.next() == true) {
			if (curDoc != spans.doc()) {
				//System.out.println("TSDoc2OccurrencesCount::fill() curDoc="+spans.doc());
				if (curDoc != -1) {
					this.put(curDoc, occCount);
				}
				occCount = this.get(spans.doc());
				// if the document is not yet in the map
				if (occCount == null) {
					occCount = 0;
				}
				curDoc = spans.doc();
			}
			occCount++;
		}
		this.put(curDoc, occCount); // do not forget about the last document
	}

	/** Accumulate counts and merge overlapping occurences.
	 */
	public void fill(Spans spans) throws IOException
	{
		int curDoc = -1;
		Integer occCount = null;
		int occprev_start = -1;
		int occprev_end = -1;
		while (spans.next() == true) {
			if (curDoc != spans.doc()) {
				//System.out.println("doc="+curDoc+" count="+occCount);
				//System.out.println("TSDoc2OccurrencesCount::fill() curDoc="+spans.doc());
				if (curDoc != -1) {
					this.put(curDoc, occCount);
				}
				occCount = this.get(spans.doc());
				// if the document is not yet in the map
				if (occCount == null) {
					occCount = 0;
				}
				occprev_start = -1;
				curDoc = spans.doc();
			}
			// THE FOLLOWING CODE WORKS ONLY IF SPANS ARE SORTED BY POSITION (TIME) !!!
			// if the current span does not overlap with the previous one, add it
			if (occprev_start == -1 || occprev_start > spans.end() || occprev_end < spans.start()) {
				occprev_start = spans.start();
				occprev_end = spans.end();
				occCount++;
			} else {
				// occprev = min(start), max(end), max(score)
				if (occprev_start > spans.start()) {
					occprev_start = spans.start();
				}
				if (occprev_end   < spans.end()) {
					occprev_end   = spans.end();
				}
			}
		}
		this.put(curDoc, occCount); // do not forget about the last document
	}
}


