package org.apache.lucene.timeslots;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.NumericField;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.Term;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.NIOFSDirectory;

import org.apache.lucene.timeslots.TSAnalyzer;
import org.apache.lucene.timeslots.TSSimilarity;

import java.io.IOException;
import java.io.FileReader;
import java.io.File;

public class TSIndexer {

	protected TSSimilarity similarity;

	protected float slotLength = 0.5f;
	protected TSAnalyzer analyzer;
	protected Directory index;
	protected IndexWriter writer;

	public TSIndexer(String indexPath, boolean createIndex)  throws IOException
	{
		analyzer = new TSAnalyzer();

		// 1. create the index
		//Directory index = new RAMDirectory();
		index = new NIOFSDirectory(new File(indexPath));

		// the boolean arg in the IndexWriter ctor means to
		// create a new index, overwriting any existing index
		writer = new IndexWriter(index, analyzer, createIndex, IndexWriter.MaxFieldLength.UNLIMITED);

		similarity = new TSSimilarity();
		writer.setSimilarity(similarity);
	}

	public void index(String docName, String docPath, float lengthSeconds) throws IOException
	{
		System.out.println("indexing '"+docName+"' -> '"+docPath+"'");
		Document doc = new Document();
		//doc.add(new Field("category", categoryName, Field.Store.YES, Field.Index.NOT_ANALYZED_NO_NORMS));

		//Store the document's name
		doc.add(new Field("docname", docName, Field.Store.YES, Field.Index.NOT_ANALYZED_NO_NORMS));
		//Store both position and offset information
		doc.add(new Field("body", new FileReader(new File(docPath)), Field.TermVector.WITH_POSITIONS_OFFSETS));
		//Store the document's length in seconds
		NumericField nf = new NumericField("length", 1, Field.Store.YES, true);
		nf.setIntValue((int)lengthSeconds);
		doc.add(nf);
		//writer.addDocument(doc);
		writer.updateDocument(new Term("docname", docName), doc);
	}

	public void commit() throws IOException, CorruptIndexException
	{
		writer.commit(); // commit() is a costly operation and should not be called for each document but after indexing a whole bunch of documents
	}

	public void close() throws IOException, CorruptIndexException
	{
		writer.close();
	}

	public void optimize() throws IOException, CorruptIndexException
	{
		System.out.println("optimize() ...");
		writer.optimize();
		System.out.println("optimize() ...done");
	}
}

