package org.apache.lucene.timeslots;

import org.apache.lucene.index.Term;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.TermQuery;
import org.apache.lucene.search.BooleanQuery;
import org.apache.lucene.search.BooleanClause;
import org.apache.lucene.search.payloads.PayloadTermQuery;
import org.apache.lucene.search.payloads.PayloadNearQuery;
//import org.apache.lucene.search.payloads.AveragePayloadFunction;
import org.apache.lucene.search.payloads.PayloadFunction;
import org.apache.lucene.search.spans.SpanQuery;
import org.apache.lucene.search.spans.SpanOrQuery;
import org.apache.lucene.search.spans.SpanTermQuery;
import org.apache.lucene.search.spans.SpanNearQuery;

import java.util.List;
import java.util.Vector;
import java.util.ArrayList;
import java.util.Set;
import java.util.HashSet;
import java.io.PrintWriter;

import org.apache.lucene.timeslots.TSWordsMap;
import org.apache.lucene.timeslots.TSWordClusters;
import org.apache.lucene.timeslots.BoostingNearQuery;
import org.apache.lucene.timeslots.TSNearQuery;
import org.apache.lucene.timeslots.TSMinPayloadFunction;

public class TSQuery {

	protected SpanQuery        mQueryBody;
	protected TermQuery        mQueryDocument;
	protected BooleanQuery     mQueryDocuments;
	protected Query            mQuery;
	protected Set<String>      mStopWordsList;
	protected int              mMaxWordDistance;
	protected PrintWriter      mOut;
	protected TSWordsMap       mWordsMap;
	protected TSWordClusters   mWordClusters;

	public TSQuery(PrintWriter out) 
	{
		mOut = out;
		mQueryBody = null;
		mQueryDocument = null;
		mQueryDocuments = null;
		mQuery = null;
		mStopWordsList = null;
		mMaxWordDistance = 0;
		mWordsMap = null;
		mWordClusters = null;
	}

	public void setStopWordsList(Set<String> stopwords) { mStopWordsList = stopwords; }
	public void setMaxWordDistance(int d) { mMaxWordDistance = d; }
	public void setWordsMap(TSWordsMap wmap) { mWordsMap = wmap; }
	public void setWordClusters(TSWordClusters wclust) { mWordClusters = wclust; }

	public SpanQuery getBodyQuery() { return mQueryBody; }
	public Query getQuery() { return mQuery; }

	public void parse(String queryStr, List<String> docList)
	{
		mOut.println("<query>");
		mOut.println("  <user>"+queryStr+"</user>");
		//PayloadFunction payloadFunction = new AveragePayloadFunction();
		PayloadFunction payloadFunction = new TSMinPayloadFunction();

		// parse queryStr
		String[] words = queryStr.split(" +");
		ArrayList<SpanQuery> clauses = new ArrayList<SpanQuery>();
		mOut.println("  <words>");
		for(int i=0; i < words.length ; i++) {
			words[i] = words[i].replaceAll("\"", "");
			if (mStopWordsList != null && mStopWordsList.contains(words[i])) {
				mOut.println("    <stopword>"+words[i]+"</stopword>");
			} else {
				ArrayList<SpanQuery> clauses_cluster = expandWordToOtherForms(words[i]);
				if (clauses_cluster != null) {
					SpanQuery[] clauses_cluster_a = clauses_cluster.toArray(new SpanQuery[0]);
					//System.out.println("adding SpanOrQuery to clauses");
					clauses.add(new SpanOrQuery(clauses_cluster_a));
				} else {
					//clauses.add(new PayloadTermQuery(new Term("body", words[i]), payloadFunction, true));
					clauses.add(new SpanTermQuery(new Term("body", words[i])));
				}
			}
		}
		mOut.println("  </words>");
		if (clauses.size() == 1) {
			mQueryBody = clauses.get(0);
		} else if (clauses.size() > 1) {
			//PayloadTermQuery[] clauses_a = clauses.toArray(new PayloadTermQuery[0]);
			SpanQuery[] clauses_a = clauses.toArray(new SpanQuery[0]);
			mQueryBody = new SpanNearQuery(clauses_a, mMaxWordDistance, false, false); // the 3rd parameter has to be FALSE to allow the whole phrase to be located in one slot
			//mQueryBody = new PayloadNearQuery(clauses_a, mMaxWordDistance, false, payloadFunction); // the 3rd parameter has to be FALSE to allow the whole phrase to be located in one slot
			//mQueryBody = new BoostingNearQuery(clauses_a, mMaxWordDistance, false); // the 3rd parameter has to be FALSE to allow the whole phrase to be located in one slot
		} else {
			mQueryBody = null;
		}
		mQuery = null;

		// constrain by a list of documents (SHOULD BE RATHER DONE BY A FILTER)
		if (mQueryBody != null && docList != null && !docList.isEmpty())
		{
			BooleanQuery q = new BooleanQuery();
			if (docList.size() == 1) {
				mQueryDocument = new TermQuery(new Term("docname", docList.get(0)));
				q.add(mQueryDocument, BooleanClause.Occur.MUST);
			} else {
				mQueryDocuments = new BooleanQuery();
				for (String docName : docList) {
					//System.out.println("restricting search to document: "+docName);
					mQueryDocuments.add(new TermQuery(new Term("docname", docName)), BooleanClause.Occur.SHOULD);
				}
				q.add(mQueryDocuments, BooleanClause.Occur.MUST);
			}
			q.add(mQueryBody, BooleanClause.Occur.MUST);
			mQuery = q;
		} else {
			mQuery = mQueryBody;
		}

		if (mQuery != null) {
			mOut.println("  <system>"+mQuery.toString()+"</system>");
		}
		mOut.println("</query>");
	}

	protected ArrayList<SpanQuery> expandWordToOtherForms(String word) 
	{
		HashSet<String> word_forms = new HashSet<String>();

		if (mWordsMap != null) {
			//expandWordByWordsMap(word, word_forms);
			Vector<String> mapped_words = mWordsMap.getWords(word);
			if (mapped_words != null) {
				//System.out.println("mapped_words.size() = "+mapped_words.size());
				for(String map_to_word : mapped_words) {
					//System.out.println("map_to_word: "+word+" -> "+map_to_word);
					if (!expandWordByWordClusters(map_to_word, word_forms)) {
						word_forms.add(map_to_word);
					}
				}
			}
		}
		if (word_forms.isEmpty()) {
			if (!expandWordByWordClusters(word, word_forms)) {
				return null;
			}
		}

		ArrayList<SpanQuery> clauses_cluster = new ArrayList<SpanQuery>();
		mOut.print("    <expandedword str='"+word+"'>");
		for(String word_form : word_forms) {
			//System.out.println("clauses_cluster.add(\""+word_form+"\")");
			mOut.print(word_form+" ");
			//clauses_cluster.add(new SpanTermQuery(new Term("body", word_form)));
			clauses_cluster.add(new PayloadTermQuery(new Term("body", word_form), new TSMinPayloadFunction(), true));
		}
		mOut.println("</expandedword>");

		return clauses_cluster;
	}

	/** Expand a given word by mWordClusters. 
	 * @return true if the word was found in clusters, otherwise false.
	 */
	protected boolean expandWordByWordClusters(String word, HashSet<String> word_forms)
	{
		if (mWordClusters != null) {
			Vector<Vector<String>> clusters_list = mWordClusters.getClusters(word);
			if (clusters_list != null) {
				//System.out.println("clusters_list.size() = "+clusters_list.size());
				for(Vector<String> cluster : clusters_list) {
					//System.out.println("cluster.size() = "+cluster.size());
					for(String map_to_word : cluster) {
						//System.out.println("map_to_word: "+word+" -> "+map_to_word);
						word_forms.add(map_to_word);
					}
				}
				return true;
			}
		}
		return false;
	}
}

