package org.apache.lucene.timeslots;

import org.apache.lucene.analysis.payloads.PayloadHelper;
import org.apache.lucene.analysis.WordlistLoader;

//import org.apache.lucene.index.IndexReader;
//import org.apache.lucene.index.TermVectorMapper;
//import org.apache.lucene.index.TermVectorOffsetInfo;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.TopDocs;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.TermsFilter;
import org.apache.lucene.index.Term;

//import org.apache.lucene.search.spans.NearSpansOrdered;
//import org.apache.lucene.search.spans.NearSpansUnordered;
import org.apache.lucene.search.spans.Spans;
import org.apache.lucene.search.spans.SpanQuery;

import org.apache.lucene.document.Document;

import org.apache.lucene.store.Directory;
import org.apache.lucene.store.NIOFSDirectory;

import org.apache.lucene.timeslots.TSAnalyzer;
import org.apache.lucene.timeslots.TSSimilarity;
import org.apache.lucene.timeslots.TSQuery;
import org.apache.lucene.timeslots.TSWordsMap;
import org.apache.lucene.timeslots.TSWordClusters;
//import org.apache.lucene.timeslots.TSDoc2OccurrencesCount;

import java.io.*;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Vector;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.Date;
import java.util.Set;

public class TSSearcher {

	protected TSSimilarity mSimilarity;

	protected float           mTimeSlotLength = 0.5f;
	protected TSAnalyzer      mAnalyzer;
	protected Directory       mIndex;
	protected IndexSearcher   mSearcher;
	protected Set<String>     mStopWordsList = null;
	protected TSWordsMap      mWordsMap = null;
	protected TSWordClusters  mWordClusters = null;
	protected boolean         mDoExplainResults = false;

	public TSSearcher(String indexPath, float mTimeSlotLength) throws IOException
	{
		System.out.println("TSSearcher constructor");
		this.mTimeSlotLength = mTimeSlotLength;

		mAnalyzer = new TSAnalyzer();

		// 1. create the index
		//mIndex = new RAMDirectory();
		mIndex = new NIOFSDirectory(new File(indexPath));

		mSearcher = new IndexSearcher(mIndex, true);

		mSimilarity = new TSSimilarity();
		mSearcher.setSimilarity(mSimilarity);
	}

	public void setDoExplainResults(boolean explain) { mDoExplainResults = explain; }


	public void loadWordsMap(String filename) throws IOException { 
		mWordsMap = new TSWordsMap();
		mWordsMap.load(filename); 
	}

	public void loadWordClusters(String filename) throws IOException { 
		mWordClusters = new TSWordClusters();
		mWordClusters.load(filename); 
	}

	public void loadStopWordsList(String filename) throws IOException
	{
		//WordlistLoader wll = new WordlistLoader();
		mStopWordsList = WordlistLoader.getWordSet(new File(filename));
	}

	public void search(
			String queryStr,        ///< query
			List<String> docList,   ///< list of documents to search in. If set to null, then searching in all documents
			boolean docListOnly,    ///< return only a list of documents without occurrences
			int docPageSize,        ///< number of document results per page
			int docPageNumber,      ///< which page of document results to output (starting from nr. 1)
			int occPageSize,        ///< number of occurrences per page
			int occPageNumber,      ///< which page of occurrences to output (starting from nr. 1)
			boolean sortOccurrencessByScore, ///< sort the list of occurrences for each document by time
			float maxWordDistanceSeconds, ///< maximum allowed distance of query words in seconds
			PrintWriter out         ///< output the results here
			)
		throws IOException
	{
		Date timerStart_searchtotal = new Date();
		System.out.println("searching for '"+queryStr+"'");
	
		byte[] queryBytes = queryStr.getBytes();
		for (int i=0; i<queryBytes.length; i++) {
			System.out.printf("%c  ", (char)queryBytes[i]);
		}
		System.out.println("");
		for (int i=0; i<queryBytes.length; i++) {
			System.out.printf("%X ", queryBytes[i]);
		}
		System.out.println("");

		if (queryStr == null) {
			return;
		}
		out.println("<?xml version=\"1.0\" encoding=\"utf8\"?>");
		out.println("<searcher>");

		int maxWordDistance = (int)(maxWordDistanceSeconds / mTimeSlotLength);

		TSQuery query = new TSQuery(out);
		query.setMaxWordDistance(maxWordDistance);
		query.setStopWordsList(mStopWordsList);
		query.setWordsMap(mWordsMap);
		query.setWordClusters(mWordClusters);
		//query.parse(queryStr, docList);
		Date timerStart = new Date();
		query.parse(queryStr, null); // docList is filtered using TermsFilter
		Date timerEnd = new Date();
		out.println("<query_parsing_time>"+((float)(timerEnd.getTime()-timerStart.getTime())/1000)+"</query_parsing_time>");
		if (query.getBodyQuery() == null) {
			out.println("<msg>Empty query</msg>");
		} else {
			//System.out.println("");
			//System.out.println("-----------------SEARCH START---------------------");
			// search the query
			TopDocs topDocs = null;
			timerStart = new Date();
			if (docList == null) {
				topDocs = mSearcher.search(query.getQuery(), docPageSize * docPageNumber);
			} else {
				topDocs = mSearcher.search(query.getQuery(), getDocsFilter(docList), docPageSize * docPageNumber);
			}
			timerEnd = new Date();
			//System.out.println("------------------SEARCH END----------------------");
			//System.out.println("");
			out.println("<searching_time>"+((float)(timerEnd.getTime()-timerStart.getTime())/1000)+"</searching_time>");

			out.println("<docpage size='"+docPageSize+"'>"+docPageNumber+"</docpage>");
			out.println("<occpage size='"+occPageSize+"'>"+occPageNumber+"</occpage>");

			timerStart = new Date();
			// print results
			printResults(out, query.getBodyQuery(), topDocs, docListOnly, docPageSize, docPageNumber, occPageSize, occPageNumber, sortOccurrencessByScore);
			timerEnd = new Date();
			out.println("<printing_results_time>"+((float)(timerEnd.getTime()-timerStart.getTime())/1000)+"</printing_results_time>");

			if (mDoExplainResults) {
				explainResults(mSearcher, query.getBodyQuery(), topDocs);
			}
		}
		Date timerEnd_searchtotal = new Date();
		out.println("<total_time>"+((float)(timerEnd_searchtotal.getTime()-timerStart_searchtotal.getTime())/1000)+"</total_time>");

		out.println("</searcher>");

// COMMENTED SNIPPETS {{{		
/*
    IndexReader reader = mSearcher.getIndexReader();
    PayloadSpanUtil psu = new PayloadSpanUtil(reader);
    
    Collection payloads = psu.getPayloadsForQuery(new TermQuery(new Term(PayloadHelper.FIELD, "rr")));
    if(DEBUG)
      System.out.println("Num payloads:" + payloads.size());
    Iterator it = payloads.iterator();
    while(it.hasNext()) {
      byte[] bytes = (byte[]) it.next();
      if(DEBUG)
        System.out.println(new String(bytes));
    }
*/


/*
		// PRINT CONTEXT
		WindowTermVectorMapper tvm = new WindowTermVectorMapper();
		int window = 2;//get the words within two of the match, inclusive of the boundaries
		while (spans.next() == true) {
			System.out.println("Doc: " + spans.doc() + " Start: " + spans.start() + " End: " + spans.end());
			//build up the window
			tvm.start = spans.start() - window;
			tvm.end = spans.end() + window;
			reader.getTermFreqVector(spans.doc(), "body", tvm);
			for (WindowEntry entry : tvm.entries.values()) {
				System.out.println("Entry: " + entry);
			}
			//clear out the entries for the next round
			tvm.entries.clear();
		}
*/
// }}}
	}

	public TermsFilter getDocsFilter(List<String> docList)
	{
		TermsFilter tf = new TermsFilter();
		for (String docName : docList) {
			//System.out.println("restricting search to document: "+docName);
			tf.addTerm(new Term("docname", docName));
		}
		return tf;
	}

	public void close() throws IOException
	{
		mSearcher.close();
	}

	private void printResults(
			PrintWriter out, 
			SpanQuery q, 
			TopDocs topDocs, 
			boolean docListOnly, 
			int docPageSize, 
			int docPageNumber,
			int occPageSize, 
			int occPageNumber,
			boolean sortOccurrencessByScore
			) throws IOException 
	{
		//System.out.println("documents found: "+topDocs.scoreDocs.length);
		out.println("<documents found='"+topDocs.totalHits+"'>");
//		topDocs = mSearcher.search(q, 10);
		
		int docs_output_index_start = docPageSize * (docPageNumber-1);
		int docs_output_index_end = Math.min(docPageSize * docPageNumber, topDocs.scoreDocs.length);

		if (docListOnly) {
			//TSDoc2OccurrencesCount doc2occurrencescount = new TSDoc2OccurrencesCount();
			//Spans spans = q.getSpans(mSearcher.getIndexReader());
			//doc2occurrencescount.fill(spans);
			//doc2occurrencescount.fill_approximate(spans);
			for (int i = docs_output_index_start; i < docs_output_index_end; i++) {
				ScoreDoc scoreDoc = topDocs.scoreDocs[i];
				Document doc = mSearcher.doc(scoreDoc.doc);
				out.println("  <doc name='" + doc.get("docname") + "' doc_id='" + scoreDoc.doc + "' length='"+doc.get("length")+"' score='" + scoreDoc.score + "'/>");
				//out.println("  <doc name='" + doc.get("docname") + "' doc_id='" + scoreDoc.doc + "' length='"+doc.get("length")+"' score='" + scoreDoc.score + "' occurrences='" + doc2occurrencescount.get(scoreDoc.doc) + "'/>");
				//out.println("  <doc name='" + doc.get("docname") + "' doc_id='" + scoreDoc.doc + "' length='"+doc.get("length")+"' score='" + scoreDoc.score + "'/>");
			}
		} else {
//			mSearcher.getSimilarity().tf(0.5f);
			HashSet<Integer> resultsDocsSet = new HashSet<Integer>(docPageSize);
			for (int i = docs_output_index_start; i < docs_output_index_end; i++) {
				resultsDocsSet.add(topDocs.scoreDocs[i].doc);
			}

			TSDoc2Occurrences doc2occurrences = new TSDoc2Occurrences();
			Spans spans = q.getSpans(mSearcher.getIndexReader());
			doc2occurrences.fill(spans, resultsDocsSet, true);

			for (int i = docs_output_index_start; i < docs_output_index_end; i++) {
				ScoreDoc scoreDoc = topDocs.scoreDocs[i];
				Document doc = mSearcher.doc(scoreDoc.doc);
				TSOccurrencesList occlist = doc2occurrences.get(scoreDoc.doc);

				//if (occlist == null) { continue; }
				out.println("  <doc name='"+doc.get("docname")+"' doc_id='"+scoreDoc.doc+"' length='"+doc.get("length")+"' score='"+scoreDoc.score+"' occurrences='"+occlist.size()+"'>");

				if (sortOccurrencessByScore) {
					Collections.sort(occlist);
				}

				int occs_output_index_start = occPageSize * (occPageNumber-1);
				int occs_output_index_end = Math.min(occPageSize * occPageNumber, occlist.size());

				//Iterator<TSOccurrence> it = occlist.iterator();
				//while (it.hasNext()) {
				//	TSOccurrence occ = it.next();
				for (int j = occs_output_index_start; j < occs_output_index_end; j++) {
					TSOccurrence occ = occlist.get(j);
					out.println("    <occ start='" + occ.timeSlotStart*mTimeSlotLength + "' end='" + occ.timeSlotEnd*mTimeSlotLength + "' score='" + occ.score + "'/>");
				}
				out.println("  </doc>");
			}
		} // if !doclistonly
		out.println("</documents>");
	}

	private void explainResults(IndexSearcher searcher, Query query, TopDocs topDocs) throws IOException {
		System.out.println("--------------------------------------------------");
		System.out.println("explainResults()");
		System.out.println("--------------------------------------------------");
		System.out.println("Results for " + query + " of type: " + query.getClass().getName());
		for (int i = 0; i < topDocs.scoreDocs.length; i++) {
			ScoreDoc doc = topDocs.scoreDocs[i];
			System.out.println("Doc: docname=" + searcher.doc(doc.doc).get("docname") + " " + doc.toString());
			System.out.println("Explain: " + searcher.explain(query, doc.doc));
		}
		System.out.println("--------------------------------------------------");
		System.out.println("explainResults() done");
		System.out.println("--------------------------------------------------");
	}
}

class TSOccurrence implements Comparable<TSOccurrence>
{
	public int timeSlotStart;
	public int timeSlotEnd;
	public float score;

	public boolean equals(Object o) {
		if (!(o instanceof TSOccurrence))
			return false;
		TSOccurrence occ = (TSOccurrence)o;
		return occ.timeSlotStart == timeSlotStart &&
			occ.timeSlotEnd == timeSlotEnd &&
			occ.score == score;
	}

	public int compareTo(TSOccurrence occ) {
		return (score < occ.score ? 1 : score > occ.score ? -1 : 0);
	}
}

class TSOccurrencesList extends Vector<TSOccurrence>
{
	private static final long serialVersionUID = 1L;
}

class TSDoc2Occurrences extends HashMap<Integer, TSOccurrencesList>
{
	private static final long serialVersionUID = 2L;

/* processSubSpans COMMENTED OUT {{{
	private void processSubSpans(Spans[] subSpans, int depth) 
	{
		try {
			System.out.println("--------------------------------------------------");
			System.out.println("processSubSpans() depth: "+depth);
			System.out.println("subSpans.size(): "+subSpans.length);
			for (Spans spans : subSpans) {
				Collection<byte[]> payloads = spans.getPayload();

				System.out.println("processSubSpans() depth="+depth+" payloads.size():"+payloads.size());
				for (final byte[] payload : payloads) {
					float payloadScore = PayloadHelper.decodeFloat(payload, 0);
					System.out.println("  payloadScore: "+payloadScore);
					//System.out.println("payload:"+payloadScore);
				}

				if (spans instanceof NearSpansOrdered) {
					processSubSpans(((NearSpansOrdered) spans).getSubSpans(), depth+1);
				} else if (spans instanceof NearSpansUnordered) {
					processSubSpans(((NearSpansUnordered) spans).getSubSpans(), depth+1);
				}
			}
		} catch (IOException e) {
			System.err.println("An exception occurred in processSubSpans(): "+e); 
		}
	}
*/ // }}}

	public void fill(Spans spans, HashSet<Integer> resultsDocsSet, boolean mergeOverlappedResults) throws IOException
	{
		int curDoc = -1;
		TSOccurrencesList occlist = null;
		TSOccurrence occprev = null;
		boolean epsilon = false;
		while (epsilon || (!epsilon && spans.next()))
		{
			if (epsilon) { epsilon = false; }
			// if the docID of the current span is not in the desired documents set, skip to the next document
			if (!resultsDocsSet.contains(spans.doc())) {
				if (!spans.skipTo(spans.doc()+1)) {
					break; // if there are no more documents, break the loop
				}
				epsilon = true;
				continue;
			}

			Collection<byte[]> payloads = spans.getPayload();
			//System.out.println("payloads.size():"+payloads.size());

			float spanScore = 1;
			//System.out.println("compute paylode score");
			for (Iterator<byte[]> iterator = payloads.iterator(); iterator.hasNext();) {
				byte[] thePayload = iterator.next();
				float payloadScore = PayloadHelper.decodeFloat(thePayload, 0);
				//System.out.println("  payloadScore: "+payloadScore);
				if (spanScore > payloadScore) {
					spanScore = payloadScore;
				}
			}
			//System.out.println("    final spanScore: "+spanScore);

			if (curDoc != spans.doc()) {
				//System.out.println("TSDoc2Occurrences::fill() curDoc="+spans.doc());
				occlist = this.get(spans.doc());
				// if the document is not yet in the map
				if (occlist == null) {
					occlist = new TSOccurrencesList();
					this.put(spans.doc(), occlist);
				}
				occprev = null;
			}
			// THE FOLLOWING CODE WORKS ONLY IF SPANS ARE SORTED BY POSITION (TIME) !!!
			// if the current span does not overlapp with the previous one, add it
			if (!mergeOverlappedResults || occprev == null || occprev.timeSlotStart > spans.end() || occprev.timeSlotEnd < spans.start()) {
				TSOccurrence occ = new TSOccurrence();
				occ.timeSlotStart = spans.start();
				occ.timeSlotEnd = spans.end();
				occ.score = spanScore;
				occlist.add(occ);
				occprev = occ;
			} else {
				// occprev = min(start), max(end), max(score)
				if (occprev.timeSlotStart > spans.start()) {
					occprev.timeSlotStart = spans.start();
				}
				if (occprev.timeSlotEnd   < spans.end()) {
					occprev.timeSlotEnd   = spans.end();
				}
				if (occprev.score         < spanScore) {
					occprev.score         = spanScore;
				}
			}

			curDoc = spans.doc();
		}
	}
}

// COMMENTED SNIPPETS {{{		
/*
//Not thread-safe
class WindowTermVectorMapper extends TermVectorMapper {
 
  int start;
  int end;
  LinkedHashMap<String, WindowEntry> entries = new LinkedHashMap<String, WindowEntry>();
 
  public void map(String term, int frequency, TermVectorOffsetInfo[] offsets, int[] positions) {
    for (int i = 0; i < positions.length; i++) {//unfortunately, we still have to loop over the positions
      //we'll make this inclusive of the boundaries
      if (positions[i] >= start && positions[i] < end){
        WindowEntry entry = entries.get(term);
        if (entry == null) {
          entry = new WindowEntry(term);
          entries.put(term, entry);
        }
        entry.positions.add(positions[i]);
      }
    }
  }
 
  public void setExpectations(String field, int numTerms, boolean storeOffsets, boolean storePositions) {
    // do nothing for this example
    //See also the PositionBasedTermVectorMapper.
  }
 
}
 
class WindowEntry{
  String term;
  List<Object> positions = new ArrayList<Object>();//a term could appear more than once w/in a position
 
  WindowEntry(String term) {
    this.term = term;
  }
 
  @Override
  public String toString() {
    return "WindowEntry{" +
            "term='" + term + '\'' +
            ", positions=" + positions +
            '}';
  }
}
*/
// }}}

