package org.apache.lucene.timeslots;

import org.apache.lucene.analysis.TokenFilter;
import org.apache.lucene.analysis.TokenStream;
import org.apache.lucene.analysis.tokenattributes.PayloadAttribute;
import org.apache.lucene.analysis.tokenattributes.TermAttribute;
import org.apache.lucene.analysis.tokenattributes.PositionIncrementAttribute;
import org.apache.lucene.analysis.payloads.PayloadEncoder;
import org.apache.lucene.analysis.payloads.FloatEncoder;

import java.io.IOException;

public class TSTokenFilter extends TokenFilter 
{
	public static final char DEFAULT_DELIMITER = '|';
	protected char delimiter = DEFAULT_DELIMITER;
	protected TermAttribute termAtt;
	protected PayloadAttribute payAtt;
	protected PositionIncrementAttribute posAtt;
	protected PayloadEncoder floatEncoder;
	protected int currentTimeSlot = 0;

	/**
	 * Construct a token stream filtering the given input.
	 */
	protected TSTokenFilter(TokenStream input) {
		super(input);
		floatEncoder = new FloatEncoder();
		termAtt = addAttribute(TermAttribute.class);
		payAtt = addAttribute(PayloadAttribute.class);
		posAtt = addAttribute(PositionIncrementAttribute.class);
	}

	public boolean incrementToken() throws IOException {
		boolean result = false;
		if (input.incrementToken()) {
			final char[] buffer = termAtt.termBuffer();
			final int length = termAtt.termLength();
			//look for the delimiter
			int timeslot = 0;
			int i=0;
			// READ THE TIME SLOT INFO
			while (i < length) {
				if (buffer[i] == delimiter) {
					String s = new String(buffer, 0, i);
					timeslot = Integer.parseInt(s);
//					System.out.println("Slot: " + s + " = " + timeslot + " .. " + (timeslot-currentTimeSlot));
					posAtt.setPositionIncrement(timeslot - currentTimeSlot);
					currentTimeSlot = timeslot;
					break;//at this point, we know the whole piece, so we can exit.  If we don't see the delimiter, then the termAtt is the same
				}
				i++;
			}
			// READ THE TERM TEXT
			int item_beg = ++i;
			while (i < length) {
				if (buffer[i] == delimiter) {
					termAtt.setTermBuffer(buffer, item_beg, i-item_beg);
					/*
					String s = new String(buffer);
					s = s.substring(item_beg, i);
					System.out.println("Term: " + s);

					byte[] sBytes = s.getBytes();
					for (int j=0; j<sBytes.length; j++) {
						System.out.printf("%c  ", (char)sBytes[j]);
					}
					System.out.println("");
					for (int j=0; j<sBytes.length; j++) {
						System.out.printf("%X ", sBytes[j]);
					}
					System.out.println("");
					*/
					break;
				}
				i++;
			}
			// READ THE TERM'S POSTERIOR PROBABILITY
			item_beg = ++i;
			int item_end = length;
			while (i < length) {
				if (buffer[i] == delimiter) {
					item_end = i;
					break;
				}
				i++;
			}
/*			String s = new String(buffer);
			s = s.substring(item_beg, item_end);
			System.out.println("Score: " + s);
*/			payAtt.setPayload(floatEncoder.encode(buffer, item_beg, item_end-item_beg));

			result = true;
		}
		return result;
	}
}


