package org.apache.lucene.timeslots;

import java.io.*;
import java.util.HashMap;
import java.util.Vector;

/** Contains clusters of words. Words can be in one cluster for example
 * if they share a stem. Each word can be in 1..N clusters. */
public class TSWordClusters
{
	/** Mapping a word to possibly N different cluster lists. 
	 * One word can appear in more clusters. */
	protected HashMap<String,Vector<Vector<String>>> mMap = null; 

	/** Get a list of clusters for a word. */
	public Vector<Vector<String>> getClusters(String word)
	{
		if (!mMap.containsKey(word)) {
			return null;
		}
		return mMap.get(word);
	}

	/** Load clusters from file. Each line in the input file contains 
	 * a list of space-separated words which belong to one cluster. */
	public void load(String filename) throws IOException
	{
		BufferedReader in = new BufferedReader(new FileReader(filename));

		mMap = new HashMap<String,Vector<Vector<String>>>();
		int i=0;
		while (in.ready()) {
			String line = in.readLine();
			i++;
			String[] a = line.split("[ \t]+");
			if (a.length < 2) {
				throw new IOException("At least 2 words have to be in a group in '"+filename+"' at line "+i);
			}

			Vector<String> v = new Vector<String>(a.length);
			for (String word : a) {
				v.add(word);

				Vector<Vector<String>> vv = null;
				if (mMap.containsKey(word)) {
					vv = mMap.get(word);
				} else {
					vv = new Vector<Vector<String>>(1);
					mMap.put(word, vv);
				}
				vv.add(v);
			}
		}
		
		in.close();
	}
}


