package org.apache.lucene.timeslots;

import java.io.*;
import java.util.HashMap;
import java.util.Vector;

/** Mapping words 1-to-many.*/
public class TSWordsMap
{
	protected HashMap<String,Vector<String>> mMap = null;

	/** Get list of words for the given word. */
	public Vector<String> getWords(String key)
	{
		if (!mMap.containsKey(key)) {
			return null;
		}
		return mMap.get(key);
	}

	/** Load mapping from file. Each line contains lines in this format:
	 * WORD_FROM WORD_MAP_TO_1 [WORD_MAP_TO_2 [WORD_MAP_TO_3 ...]]
	 */
	public void load(String filename) throws IOException
	{
		BufferedReader in = new BufferedReader(new FileReader(filename));

		mMap = new HashMap<String,Vector<String>>();
		int line_nr=0;
		while (in.ready()) {
			String line = in.readLine();
			line_nr++;
			String[] a = line.split("[ \t]+");
			if (a.length == 1) {
				throw new IOException("Only one word in '"+filename+"' at line "+line_nr);
			}
			String key   = a[0];
			for (int i=1; i<a.length; i++) {
				String value = a[i];
				Vector<String> v = null;
				if (mMap.containsKey(key)) {
					v = mMap.get(key);
				} else {
					v = new Vector<String>();
					mMap.put(key, v);
				}
				v.add(value);
			}
			//System.out.println(key+" => "+value);
		}
		
		in.close();
	}
}

